#include <QFile>
#include <QSettings>
#include <QMimeDatabase>
#include <QTextStream>

#include "processmanager.h"

static QString cmdForMime(QString mimeStr);
static QString cmdForMime(QMimeType mime);
static QString cmdForFile(QString filePath);

RProcess::RProcess(QString cmd, QString filePath, QObject *parent)
    : QProcess (parent)
    , m_cmd(cmd)
    , m_filePath(filePath)
{
    connect(this, SIGNAL(finished(int, QProcess::ExitStatus)),
            this, SLOT(onFinished(int, QProcess::ExitStatus)));
}

void RProcess::go()
{
    start(m_cmd);
}

void RProcess::onFinished(int exitCode, QProcess::ExitStatus exitStatus)
{
    QFile f(m_filePath);
    f.remove();
}


void SubprocessManager::empty()
{
    foreach (const RProcess *process, m_processes)
    {
        delete process;
    }
    m_processes.clear();
}

bool SubprocessManager::open(QString filePath, QString mimetype)
{
    QString cmd;
    if (! mimetype.isEmpty())
        cmd = cmdForMime(mimetype);
    else
        cmd = cmdForFile(filePath);

    if (cmd.isEmpty())
        return false;

    cmd = cmd.replace("%u", filePath).replace("%U", filePath);
    cmd = cmd.replace("%f", filePath).replace("%F", filePath);

    RProcess *p = new RProcess(cmd, filePath, this);
    p->go();
    m_processes << p;

    return true;
}

static QString cmdForMime(QString mimeStr)
{
    QMimeType mime;
    QMimeDatabase db;
    mime = db.mimeTypeForName(mimeStr);

    return cmdForMime(mime);
}

static QString cmdForMime(QMimeType mime)
{
    QStringList l;

    QFile cache("/usr/share/applications/mimeinfo.cache");
    if (cache.open(QIODevice::ReadOnly))
    {
        QTextStream in(&cache);
        while (!in.atEnd())
        {
            QString line = in.readLine();
            if (line.startsWith(mime.name() + "="))
            {
                QStringList tmp = line.split("=");
                l = tmp[1].split(";");
                break;
            }
        }
    }
    cache.close();

    QString app;
    foreach (const QString &desktopFile, l)
    {
        QFile f("/usr/share/applications/" + desktopFile);
        if (!f.exists())
            continue;

        QSettings s(f.fileName(), QSettings::IniFormat);
        app = s.value("Desktop Entry/Exec").toString();
        break;
    }

    return app;
}

static QString cmdForFile(QString filePath)
{
    QMimeDatabase db;
    QMimeType mime = db.mimeTypeForFile(filePath);

    if (mime.isValid())
        return cmdForMime(mime);
    else
        return QString();
}
