#include <QVBoxLayout>
#include <QDebug>

#include "video.h"

Video::Video(PlaylistItem *item, QWidget *parent)
    : QWidget(parent)
{
    const char *const vlc_args[] = {
        "-I", "dummy",
        "--no-xlib",
        "--ignore-config",
        "--no-video-title-show",
        "--no-disable-screensaver",
    };

    m_time = item->time();
    m_uri = item->uri();

    setAutoFillBackground(true);
    QPalette plt;
    plt.setColor(QPalette::Window, Qt::black);
    setPalette(plt);

    qDebug() << "FLUP";
    vlc_inst = libvlc_new(sizeof(vlc_args) / sizeof(vlc_args[0]), vlc_args);
    qDebug() << "FLUP";
    vlc_mp = libvlc_media_player_new(vlc_inst);
    qDebug() << "FLUP";
    libvlc_event_manager_t *em = libvlc_media_player_event_manager(vlc_mp);
    qDebug() << "FLUP";
    libvlc_event_attach(em, libvlc_MediaPlayerEndReached,
                        mediaEndReached, this);

    connect(this, SIGNAL(end()), this, SLOT(onEnd()));
    qDebug() << "CONSTRUCTED";
}

Video::~Video()
{
    libvlc_media_player_stop(vlc_mp);
    libvlc_media_player_release(vlc_mp);
    libvlc_release(vlc_inst);
    m_timer.stop();
}

void Video::onEnd()
{
    m_timer.stop();
}

void Video::onTimeout()
{
    stop();
    emit(end());
}

void Video::stop()
{
    m_timer.stop();
    libvlc_media_player_stop(vlc_mp);
    libvlc_media_release(vlc_m);
}

void Video::mediaEndReached(const libvlc_event_t *e, void *param)
{
    Video *const v = static_cast<Video *>(param);
    libvlc_media_release(v->vlc_m);
    emit v->end();
}

void Video::run()
{
    vlc_m = libvlc_media_new_location(vlc_inst, m_uri.toUtf8().data());
    libvlc_media_player_set_media(vlc_mp, vlc_m);
    libvlc_media_player_set_xwindow(vlc_mp, winId());
    libvlc_media_player_play(vlc_mp);
    qDebug() << "HERE";

    if (m_time != 0)
        m_timer.start(m_time * 1000);
}

