#include <QNetworkCookie>
#include <QVariant>
#include <QSqlQuery>
#include <QDateTime>
#include <QWebElementCollection>
#include <QWebFrame>

#include <QDebug>

#include "webpage.h"

SqliteCookieJar::SqliteCookieJar(QObject *parent)
    : QNetworkCookieJar(parent)
{
    QSqlQuery query("SELECT name,value,host,path,expiry,isSecure,isHttpOnly FROM moz_cookies");
    while (query.next())
    {
        QNetworkCookie c;
        c.setName(query.value(0).toByteArray());
        c.setValue(query.value(1).toByteArray());
        c.setDomain(query.value(2).toByteArray());
        c.setPath(query.value(3).toByteArray());
        c.setExpirationDate(QDateTime::fromTime_t(query.value(4).toInt()));
        c.setSecure(query.value(5).toBool());
        c.setHttpOnly(query.value(6).toBool());
        m_cookies.append(c);
    }

    setAllCookies(m_cookies);
}


Webpage::Webpage(PlaylistItem *item, QWidget *parent)
    : QWebView(parent)
{
    m_time = item->time();
    m_uri = QUrl(item->uri());

    connect(this, SIGNAL(loadFinished(bool)), this, SLOT(onLoadFinished(bool)));

    installEventFilter(this);

    m_jar = new SqliteCookieJar(this);
    page()->networkAccessManager()->setCookieJar(m_jar);

    connect(&m_timer, &QTimer::timeout, this, &Webpage::onTimeout);
}

bool Webpage::eventFilter(QObject *obj, QEvent *event)
{
    setMouseTracking(true);
    switch (event->type())
    {
        case QEvent::MouseMove:
        case QEvent::MouseButtonPress:
        case QEvent::KeyPress:
            qobject_cast<QWidget *>(obj)->setCursor(Qt::BlankCursor);
            return true;
        default:
            return false;
    }
}

void Webpage::onLoadFinished(bool ok)
{
    if (!ok)
        emit(end());
}

void Webpage::onTimeout()
{
    qDebug() << "TIMEOUT";
    m_timer.stop();
    emit(end());
}

void Webpage::run()
{
    load(m_uri);
    m_timer.start(1000 * m_time);
}

void Webpage::stop()
{
    QWebElementCollection elements = page()->mainFrame()->findAllElements("embed");
    foreach (QWebElement e, elements)
    {
        e.removeFromDocument();
    }
    m_timer.stop();
}
